﻿-- Author      : Zaultar
-- Create Date : 11/27/2008 10:17:49 PM
-- _O1_ OnLoad()
-- _02_ OnEvent()


-- Addon constants
CN_NAME 	= "Chat Notes"
CN_VNMBR	= 303001			-- Number code for this version

-- Colors
CN_HELP_COLOR		= "|cFFFFFF7F"
CN_ERROR_COLOR	    = "|cFFFF0000"
CN_UNKNOWN_COLOR	= "|cFF7F7F7F"
CN_TOKEN_COLOR	    = "|cFFFF0000"
CN_NORMAL_COLOR     = "|r"

-- Stored Data
if not CN_State then CN_State = {} end		-- Data saved between sessions
if not CN_Notes then CN_Notes = {} end		-- Names database
if not CN_FNotes then CN_FNotes = {} end	-- Friend Notes database
if not CN_GNotes then CN_GNotes = {} end	-- Guild Notes database

-- Slash variables
local SLASH_ON = "on";			-- Enabled Chat Notes? (really?)
local SLASH_OFF = "off";		-- Enabled Chat Notes? (really?)
local SLASH_ADD = "add"			-- Add a player note
local SLASH_WHO = "whois"		-- Whois a user
local SLASH_COLOR = "color"		-- Toggles Color on/off
local SLASH_DEBUG = "debug";	-- Toggles Debug output on/off


-- _01_ OnLoad() Handler	
------------------------------------------------------------
function ChatNotes_OnLoad(self) 
	self:RegisterEvent("VARIABLES_LOADED");

	SLASH_CHATNOTES1 = "/chatnotes"
	SLASH_CHATNOTES2 = "/cn"
	SlashCmdList["CHATNOTES"] = CN_ParseSlash;
	
	SLASH_WHOIS1 = "/whois"
	SlashCmdList["WHOIS"] = CN_WhoPlayer;

	SLASH_ADD1 = "/add"
	SlashCmdList["ADD"] = CN_AddCommand;
	
	DEFAULT_CHAT_FRAME:AddMessage(CN_HELP_COLOR..CN_NAME..":|r "..CN_TXT.loaded..CN_HELP_COLOR..CN_TXT.loadedtext);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", myChatFilter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", myChatFilter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", myChatFilter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", myChatFilter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", myChatFilter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD", myChatFilter)
end

-- Filter Chat input and add our note to the begining of the string.
function myChatFilter(msg)
	if (CN_State.enabled) then
		if strfind(msg, "lol") then
		    return false, gsub(msg, "lol", "LOL")
	    end
		-- If we have a value in this table, then display the note
		if (CN_Notes[arg2] ) then
			local output 
			if (CN_State.color) then output = CN_HELP_COLOR else output = "" end
			return false, output.."("..CN_Notes[arg2]..") "..CN_NORMAL_COLOR..msg
		end
	end
end


-- _02_ OnEvent() Handler
------------------------------------------------------------
function ChatNotes_OnEvent(self, event, arg1, arg2) 
	CN_Debug("OnEvent".."self:"..self.." event:"..event.."arg1"..arg1)
	if (event == "VARIABLES_LOADED") then
		if CN_State.debug == nil then CN_State.debug = false end
		if CN_State.color == nil then CN_State.color = false end
		if CN_State.enabled == nil then CN_State.enabled = true end
		CN_DisplayStatus()
		CN_Debug("VARIABLES_LOADED")
	elseif (event == "SYSTEM") then
	    CN_Debug("arg1="..arg1.." arg2="..arg2)
	elseif (event == "PLAYER_ENTERING_WORLD") then
		CN_Debug("PLAYER_ENTERING_WORLD"..arg1)
	elseif (event == "GUILD_REGISTRAR_SHOW") then
		CN_Debug("GUILD_REGISTRAR_SHOW")
	elseif (event == "FRIENDLIST_UPDATE") then
		CN_Debug("FRIENDLIST_UPDATE")
	end
end


-- _03_ - OnEvent Functions
------------------------------------------------------------

-- Main routine for adding a Note to a specific player name
------------------------------------------------------------
function CN_AddPlayerNote (player, note)
	local previousnote = CN_Notes[player];
	if (previousnote) then
		CN_Notes[player] = note;
		DEFAULT_CHAT_FRAME:AddMessage(CN_HELP_COLOR..CN_TXT.updated..CN_NORMAL_COLOR..string.format(" %s:%s (it was: %s)",player,note,previousnote));
	else
		CN_Notes[player] = note;
		DEFAULT_CHAT_FRAME:AddMessage(string.format(CN_HELP_COLOR..CN_TXT.added..CN_NORMAL_COLOR.." %s:%s",player,note));
	end
end

-- _04_ Chat Window Functions
-----------------------------
-- Is the SYSTEM message a Login Message
------------------------
-- TODO CURRENTLY NOT USED
function CN_IsLoginMessage(msg)
	if msg then 
		if ( string.find(msg, "has come online") ~= nil ) then
			return true
		end
	end
end

------------------------
-- _05_ Slash Handler --
------------------------

-- Main Slash Command handler
------------------------------------------------------------
function CN_ParseSlash(msg) 
	local cmd, params = CN_Extract_NextParameter(msg);
	if (cmd == SLASH_ON) then	
		CN_State.enabled = true;
		CN_DisplayStatus();
	elseif (cmd == SLASH_OFF) then
		CN_State.enabled = false;
		CN_DisplayStatus();
	elseif (cmd == SLASH_ADD) then
		if (params) then
			local playername, note = CN_Extract_NextParameter(params);
			CN_AddPlayerNote (playername, note);
		else
			CN_DisplayAddUsage();
		end
	elseif (cmd == SLASH_WHO) then
		if (params) then
			CN_WhoPlayer(params);
		else
			CN_DisplayWhoUsage();
		end
	elseif (cmd == SLASH_COLOR) then
	    CN_State.color = not CN_State.color
	    CN_DisplayStatus();
	elseif (cmd == SLASH_DEBUG) then
		CN_State.debug = not CN_State.debug;
		CN_DisplayStatus();
	else 
		CN_DisplayUsage();
	end
	
end

-- Generic Add command, parses input to ensure we have a name value pair
------------------------------------------------------------
function CN_AddCommand (cmd)
	if (cmd) then
		local playername, note = CN_Extract_NextParameter(cmd);

		-- Normalize the inputed player name
		playername = CN_MixedCase(playername)
		
		if (note) then
			CN_AddPlayerNote (playername, note);
		else
			CN_DisplayAddUsage();
		end
	else
		CN_DisplayUsage();
	end
end

-- /whois handler for quick query of a username
------------------------------------------------------------
function CN_WhoPlayer (player)
	local playernote = nil
	local msg = ""

	-- Normalize the inputed player name
	player = CN_MixedCase(player)
	
	if (CN_Notes[player] or CN_FNotes[player]) then
--		DEFAULT_CHAT_FRAME:AddMessage(CN_TXT.whois..string.format(": [%s] (%s):",player,CN_Notes[player]));
		-- Check Guild, Friend, then CN_Notes for notes
		if (CN_FNotes[player]) then playernote = CN_FNotes[player] end
		if (CN_Notes[player]) then playernote = CN_Notes[player] end
	else
		local numFriends = GetNumFriends();
		local numFriendsOnline = 0;
		local name, level, class, zone, connected, status, note
		local friendIndex
		if (numFriends > 0) then
			CN_List_Friends = {};
			for friendIndex=1, numFriends do
	--			name, level, class, zone, connected = GetFriendInfo(friendIndex);
				name, level, class, area, connected, status, note = GetFriendInfo(friendIndex);
				if ( name ~= NIL ) then
					if (name == player) then
						msg = "Level "..level.." "..class.." in: "..zone
					end
				else
					DEFAULT_CHAT_FRAME:AddMessage("Error: Problem retrieving friend information")
				end
				num=friendIndex;
			end
			CN_Debug("[CN] Retrieved data for "..num.." friends")
		else
			CN_Debug("[CN] Friend info unavailable. Please open the friend window once per session.")
		end

		playernote = note
		CN_FNotes[player] = note
	end
	if (playernote) then
		CN_Print(CN_HELP_COLOR.."Whois: ["..player.."]: ("..playernote..") "..msg)
	else
		CN_Print(CN_HELP_COLOR.."Whois: ["..player.."]: "..msg)
	end	
end

-- Handles processing commandline items in single chunks
------------------------------------------------------------
function CN_Extract_NextParameter(msg)
	local params = msg;
	local command = params;
	local index = strfind(command, " ");
	if ( index ) then
		command = strsub(command, 1, index-1);
		params = strsub(params, index+1);
	else
		params = nil;
	end
	return command, params;
end


-----------------------------------
-- _06_ General Helper Functions --
-----------------------------------

------------------------------------------------------------
function CN_Print(msg, forceDefault)
	if not (msg) then return end
		DEFAULT_CHAT_FRAME:AddMessage(msg)
end

function CN_Debug(msg)
	if not (msg) then return end
	if (CN_State.debug) then
		DEFAULT_CHAT_FRAME:AddMessage(msg)
	end
end

function CN_SafeConcat2(str1, str2)
	if str1 and str2 then 
		return str1..str2
	elseif str1 then
		return str1
	elseif str2 then
		return str2
	else
		return nil
	end
end

-- Returns the string in proper Mixed Case... First letter capitalized, the rest lower
function CN_MixedCase(str)
	--return string.upper(string.sub(str,1,1))..string.sub(string.lower(str),2,-1))
	local Mixed = string.upper(str:sub(1,1))..string.sub(str:lower(),2,-1)
	--return string.upper(string.sub(str,1,1))..string.sub(string.lower(str),2,-1))
	return Mixed
end

-----------------------------------
-- _07_ Information
-----------------------------------

------------------------------------------------------------
function CN_About()
	local ver = GetAddOnMetadata("ChatNotes", "Version");
	local date = GetAddOnMetadata("ChatNotes", "X-Date");
	local author = GetAddOnMetadata("ChatNotes", "Author");
	local web = GetAddOnMetadata("ChatNotes", "X-Website");

	if (author ~= nil) then
		CN_Print(CN_NAME.." "..CN_TXT.by.." "..CN_HELP_COLOR..author.."|r", true);
	end
	if (ver ~= nil) then
		CN_Print("  "..CN_TXT.version..": "..CN_HELP_COLOR..ver.."|r", true);
	end
	if (date ~= nil) then
		CN_Print("  "..CN_TXT.date..": "..CN_HELP_COLOR..date.."|r", true);
	end
	if (web ~= nil) then
		CN_Print("  "..CN_TXT.web..": "..CN_HELP_COLOR..web.."|r", true);
	end
end


function CN_DisplayUsage()
	-- CN_Print(" ", true);
	CN_Print(CN_HELP_COLOR..CN_NAME..":|r "..CN_TXT.help, true);
	CN_Print(CN_HELP_COLOR..CN_TXT.usage..":|r "..CN_TOKEN_COLOR..SLASH_ON.."/"..SLASH_OFF..CN_HELP_COLOR.." -- "..CN_TXT.usageonoff, true);
	CN_DisplayAddUsage();
	CN_DisplayWhoUsage();
	CN_Print(CN_HELP_COLOR..CN_TXT.usage..":|r "..CN_TOKEN_COLOR..SLASH_COLOR..CN_HELP_COLOR.." -- "..CN_TXT.usagecolor, true);
	CN_Print(CN_HELP_COLOR..CN_TXT.usage..":|r "..CN_TOKEN_COLOR..SLASH_DEBUG..CN_HELP_COLOR.." -- "..CN_TXT.usagedebug, true);
end

-- DisplayAddUsage
----------------------
function CN_DisplayAddUsage()
	CN_Print(CN_HELP_COLOR..CN_TXT.usage..":|r "..CN_TOKEN_COLOR..SLASH_ADD.." <playername> <note>"..CN_HELP_COLOR.." -- "..CN_TXT.usageadd, true);
end

-- DisplayWhoUsage
function CN_DisplayWhoUsage()
	CN_Print(CN_HELP_COLOR..CN_TXT.usage..":|r "..CN_TOKEN_COLOR..SLASH_WHO.." <playername>"..CN_HELP_COLOR.." -- "..CN_TXT.usagewho, true);
end

-- Display Status
-----------------------
function CN_DisplayStatus()
	message = CN_NAME..": ";
	if (CN_State.enabled) then
		message = message..CN_NORMAL_COLOR..CN_TXT.enabled;
	else
		message = message..CN_ERROR_COLOR..CN_TXT.disabled;
	end
	
	message = message.." "..CN_HELP_COLOR..CN_TXT.colormode..": "
	if (CN_State.color) then		
		message = message..CN_NORMAL_COLOR..CN_TXT.enabled;
	else
		message = message..CN_ERROR_COLOR..CN_TXT.disabled;
	end
		
	if (CN_State.debug) then
		message = message..CN_NORMAL_COLOR.."  ##DEBUG MODE##";
	end
	DEFAULT_CHAT_FRAME:AddMessage(message);
end